-- Settings
local settingsFrame = CreateFrame("Frame", "DLUSettingsFrame");

-- Notification System
CreateFrame("Frame", "DLUNotificationFrame");
DLUNotificationFrame:SetAlpha(0);
DLUNotificationFrame:SetWidth(320);
DLUNotificationFrame:SetHeight(80);
DLUNotificationFrame:SetPoint("TOP", UIParent, "TOP", 0, 0);
DLUNotificationFrame:CreateFontString("DLUNotification", "ARTWORK", "GameFontNormalLarge");
DLUNotification:SetPoint("CENTER", 0, 0);

-- UI related functions
local function achievementCompleted(achievementId)
	local _, _, _, _, _, _, _, _, _, _, _, _, wasEarnedByMe = GetAchievementInfo(achievementId);
	return wasEarnedByMe;
end

local function addToTracker(achievementId, button)
	if (achievementCompleted(achievementId)) then
		RemoveTrackedAchievement(achievementId);
		if (button ~= nil) then
			disableButton(button, "Already completed!");
		end
	else
		AddTrackedAchievement(achievementId);
	end
end

local function removeFromTracker(achievementId)
	local isTracked = IsTrackedAchievement(achievementId);
	if (isTracked) then
		RemoveTrackedAchievement(achievementId);
	end
end

-- UI functions
local uniquealyzer = 1;
local function createCheckbutton(parent, x_loc, y_loc, displayname)
	uniquealyzer = uniquealyzer + 1;
	
	local checkbutton = CreateFrame("CheckButton", "my_addon_checkbutton_0" .. uniquealyzer, parent, "ChatConfigCheckButtonTemplate");
	checkbutton:ClearAllPoints()
	checkbutton:SetPoint("TOPLEFT", x_loc, y_loc);
	getglobal(checkbutton:GetName() .. 'Text'):SetText(displayname);

	return checkbutton;
end

function DLU.createTextRoot(text, parent, x_loc, y_loc, fontType)
	local title = parent:CreateFontString(nil, "ARTWORK", fontType);
	title:SetPoint("TOPLEFT", x_loc, y_loc);
	title:SetText(text);
	return title;
end

function DLU.createTextChild(text, parent, below, x_loc, y_loc, fontType)
	local textChildNode = parent:CreateFontString(nil, "ARTWORK", fontType);
	textChildNode:SetPoint("TOPLEFT", below, "BOTTOMLEFT", x_loc, y_loc);
	textChildNode:SetText(text);
	return textChildNode;
end

local function createButton(parent, text, width, height, tooltip, x_loc, y_loc)
	local button = CreateFrame("Button", nil, parent, "UIPanelButtonTemplate");
	button:SetPoint("TOPLEFT", parent, "BOTTOMLEFT", x_loc, y_loc);
    button:SetText(text);
    button.tooltipText = tooltip;
    button:SetWidth(width);
    button:SetHeight(height);
	return button;
end

local function disableButton(button, text)
	button:SetText(text);
	button:Disable();
end

local function SetUpAddonOptions()
	local panel = CreateFrame("Frame", "DLUOptionsPanel", InterfaceOptionsFramePanelContainer);
	-- Option panel name
	panel.name = DLU.addonName;
	-- Add info to panel
	local title = DLU.createTextRoot(DLU.addonName, panel, 16, -16, "GameFontNormalLarge");
	local authorText = DLU.createTextChild("Made by: " .. GetAddOnMetadata(DLU.addonName, "Author"), panel, title, 0, -8, "GameFontHighlightSmall");
	local versionText = DLU.createTextChild("Version: ".. DLU.addonVersion, panel, authorText, 0, -8, "GameFontHighlightSmall");
	local descriptionText = DLU.createTextChild("Description: " .. GetAddOnMetadata(DLU.addonName, "X-Notes"), panel, versionText, 0, -8, "GameFontHighlightSmall");
	local websiteText = DLU.createTextChild("Author website: " .. GetAddOnMetadata(DLU.addonName, "X-Website") .. " (it's on Danish)", panel, descriptionText, 0, -8, "GameFontHighlightSmall");
	-- Add help button
	local helpButton = createButton(panel, "Help", 80, 22, "Shows a list of commands, that you can use.", 10, 30);
	helpButton:SetScript("OnClick", function()
		SlashCmdList.DLU("help");
	end);
	-- Add panel to addon options
	InterfaceOptions_AddCategory(panel);
	
	-- SettingsPanel
	local settingsPanel = CreateFrame("Frame", "DLUSettings", UIParent);
	settingsPanel.name = "Settings";
	settingsPanel.parent = panel.name;
	InterfaceOptions_AddCategory(settingsPanel);
	-- SettingsPanel text
	local settingsPanelTitle = DLU.createTextRoot(settingsPanel.name, settingsPanel, 16, -16, "GameFontNormalLarge");
	local settingsPanelDescription = DLU.createTextChild(format("Here you can change %s settings.", DLU.addonNameColored), settingsPanel, settingsPanelTitle, 0, -8, "GameFontHighlightSmall");
	-- Setting buttons
	local checkX = -60;
	for t, s in pairs(DLUSettings) do
		local checkBox = createCheckbutton(settingsPanel, 10, checkX, format("%s option", t));
		checkBox:SetChecked(DLUSettings[t]);
		checkBox:SetScript("OnClick", function()
			local settingText;
			
			if (checkBox:GetChecked()) then
				DLUSettings[t] = true;
				settingText = DLU.colorizeString("enabled", "green");
			else
				DLUSettings[t] = false;
				settingText = DLU.colorizeString("disabled", "red");
			end
			
			DLU.createMessage(format("%s option %s!", t, settingText));
		end);
		
		checkX = checkX - 20;
	end
	
	-- Artifact panel
	local artifactPanel = CreateFrame("Frame", "dluArtifactOptionPanel", UIParent);
	artifactPanel.name = "Artifact";
	artifactPanel.parent = panel.name;
	InterfaceOptions_AddCategory(artifactPanel);
	-- Artifact panel text
	local artiTitle = DLU.createTextRoot(artifactPanel.name, artifactPanel, 16, -16, "GameFontNormalLarge");
	local artiDescription = DLU.createTextChild("This page will have some buttons to help you track artifact achievements.", artifactPanel, artiTitle, 0, -8, "GameFontHighlightSmall");
	-- Hidden artifact	
	local hiddenArtifactAchievementTable = {
		["Dungeons completed"] = 11152,
		["World Quests completed"] = 11153,
		["Honorable kills"] = 11154
	};
	
	local buttonX = 10;
	for t, q in pairs(hiddenArtifactAchievementTable) do
		local trackText = "Track " .. t;
		local untrackText = "Untrack " .. t;
		local button = createButton(artifactPanel, trackText, 200, 22, "Tracks " .. t .. " needed for hidden artifact weapon appearance.", buttonX, 30);
		if (achievementCompleted(q)) then
			disableButton(button, "Already completed!");
		elseif (IsTrackedAchievement(q)) then
			button:SetText(untrackText);
		end
		button:SetScript("OnClick", function()
			local isTracked = IsTrackedAchievement(q);
			if (isTracked) then
				button:SetText(trackText);
				removeFromTracker(q);
			else
				button:SetText(untrackText);
				addToTracker(q);
			end
		end);
		buttonX = buttonX + 200;
	end
	
	-- Class specific trackers --
	local palaPanel = CreateFrame("Frame", "DLUPalaPanel", UIParent);
	palaPanel.name = DLU.getClassColored("Paladin", "PALADIN");
	palaPanel.parent = artifactPanel.name;
	InterfaceOptions_AddCategory(palaPanel);
	-- Text
	local palaTitle = DLU.createTextRoot(palaPanel.name, palaPanel, 16, -16, "GameFontNormalLarge");
	local retArtiName = DLU.itemColorString(6, "Heart of Corruption");
	local palaText = DLU.createTextChild(format("%s requires Artifact Knowledge level 6 (Retribution).", retArtiName), palaPanel, palaTitle, 0, -8, "GameFontHighlightSmall");
	-- Button
	local retButton = createButton(palaPanel, format("Check %s status", retArtiName), 220, 22, "Check how far you are to unlock the Corrupted Ashbringer.", 390, 30);
	retButton:SetScript("OnClick", function()
		DLU.createMessage("Not the exact order!", "error"); -- other color (not really an error)
		
		local steps = {
			["Has talked to Prince Tortheldrin"] = 43682;
		};
		
		if (DLU.player.faction == "Horde") then
			steps["Has talked to Bardu"] = 43683;
		else
			steps["Has talked to Alexia"] = 43683;
		end
		
		steps["Can slime drop Timolain"] = 43684;
		steps["Can Shard be fished up"] = 43685;
		
		DLU.tableStatus(steps);
	end);
	
	-- Mage --
	local magePanel = CreateFrame("Frame", "DLUMagePanel", UIParent);
	magePanel.name = DLU.getClassColored("Mage", "MAGE");
	magePanel.parent = artifactPanel.name;
	InterfaceOptions_AddCategory(magePanel);
	-- Text
	local mageTitle = DLU.createTextRoot(magePanel.name, magePanel, 16, -16, "GameFontNormalLarge");
	local arcArtiName = DLU.itemColorString(6, "The Woolomancer's Charge");
	local mageText = DLU.createTextChild(format("%s required level 6 Artifact Knowledge (Arcane).", arcArtiName), magePanel, mageTitle, 0, -8, "GameFontHighlightSmall");
	-- Button
	local arcButton = createButton(magePanel, format("Check %s status", arcArtiName), 260, 22, "Check how far you are to unlock the Woolomancer's Charge.", 350, 30);
	arcButton:SetScript("OnClick", function()
		local steps = {
			["Aszuna"] = 43787,
			["Stormheim"] = 43789,
			["Val'Sharah"] = 43790,
			["Suramar"] = 43791,
			["High Mountain"] = 43788
		}
		
		DLU.tableStatus(steps);
	end);
	
	-- Priest --
	local priestPanel = CreateFrame("Frame", "DLUPriestPanel", UIParent);
	priestPanel.name = DLU.getClassColored("Priest", "PRIEST");
	priestPanel.parent = artifactPanel.name;
	InterfaceOptions_AddCategory(priestPanel);
	-- Text
	local priestTitle = DLU.createTextRoot(priestPanel.name, priestPanel, 16, -16, "GameFontNormalLarge");
	local discArtiName = DLU.itemColorString(6, "Writings of the End");
	local priestText = DLU.createTextChild(format("%s required Artifact Knowledge 4 (Discipline).", discArtiName), priestPanel, priestTitle, 0, -8, "GameFontHighlightSmall");
	-- Button
	local discButton = createButton(priestPanel, format("Check %s Status", discArtiName), 220, 22, "Check how far you are to unlock the Writtings of the End.", 390, 30);
	discButton:SetScript("OnClick", function()
		DLU.createMessage("Not the exact order!", "error"); -- other color (not an error)
		
		local steps = {
			["Dalaran - The Violet Citadel"] = 44339,
			["Class Order Hall - Juvess the Duskwhisperer"] = 44340,
			["Northrend - New Hearthglen"] = 44341,
			["Class Order Hall - Archivist Inkforge "] = 44342,
			["Scholomance - Chillheart's Room"] = 44343,
			["Class Order Hall - Meridelle Lightspark"] = 44344,
			["Scarlet Halls - The Flameweaver's library"] = 44345,
			["Azsuna - Chief Bitterbrine Azsuna"] = 44346,
			["Suramar - Artificer Lothaire"] = 44347,
			["Black Rook Hold - Library after First Boss"] = 44348,
			["Karazhan - Guardian's Library"] = 44349,
			["Stormheim - Inquisitor Ernstonbok"] = 44350
		}
		
		DLU.tableStatus(steps);
	end);
	
	-- Warrior --
	local warPanel = CreateFrame("Frame", "DLUWarriorPanel", UIParent);
	warPanel.name = DLU.getClassColored("Warrior", "WARRIOR");
	warPanel.parent = artifactPanel.name;
	InterfaceOptions_AddCategory(warPanel);
	-- Text
	local warTitle = DLU.createTextRoot(warPanel.name, warPanel, 16, -16, "GameFontNormalLarge");	
	local protArtiName = DLU.itemColorString(6, "Burning Plate of the Worldbreaker");
	local warText = DLU.createTextChild(format("%s required Artifact Knowledge 5 (Protection).", protArtiName), warPanel, warTitle, 0, -8, "GameFontHighlightSmall");
	-- Button
	local protButton = createButton(warPanel, format("Check %s Status", protArtiName), 300, 22, "Check how far you are to unlock the Burning Plate of the Worldbreaker.", 310, 30);
	protButton:SetScript("OnClick", function()
		DLU.createMessage("Has to be available and not denied!", "error"); -- other color (not an error)
		
		local steps = {
			["Available"] = 44311,
			["Denied"] = 44312
		}
		
		DLU.tableStatus(steps);
	end);
end

-- Minimap icon
local function iconToolTip(icon)
	icon:AddLine(DLU.addonNameColored);
	icon:AddLine("Open " .. DLU.addonName .. " help page.");
	icon:AddLine("May require 2 clicks once every session.");
end

local function iconClickEvent()
	if not InterfaceOptionsFrame:IsShown() then
        InterfaceOptionsFrame_OpenToCategory(DLU.addonName);
    else
        InterfaceOptionsFrame:Hide();
    end
end

local addon = LibStub("AceAddon-3.0"):NewAddon("DLU", "AceConsole-3.0")
local DLU_LDB = LibStub("LibDataBroker-1.1"):NewDataObject("DLU", {
	type = "Launcher",
	text = "DLU_Icon",
	icon = "Interface\\AddOns\\DarkruneDKLevelUp\\Images\\dlu_icon",
	OnClick = function() iconClickEvent() end,
	OnTooltipShow = function(tooltip) iconToolTip(tooltip) end
})
local icon = LibStub("LibDBIcon-1.0");

function addon:OnInitialize()
	-- Obviously you'll need a ## SavedVariables: BunniesDB line in your TOC, duh!
	self.db = LibStub("AceDB-3.0"):New("DLU_UI_DB", {
		profile = {
			minimap = {
				hide = false,
			},
		},
	})
	icon:Register("DLU_Icon", DLU_LDB, self.db.profile.minimap)
end

-- Initialization
settingsFrame:RegisterEvent("PLAYER_LOGIN");
settingsFrame:SetScript("OnEvent", function() 

SetUpAddonOptions();

settingsFrame:UnregisterEvent("PLAYER_LOGIN")
end);